@@ -75,6 +75,7 @@ from snowflake.snowpark._internal.analyzer.grouping_set import (
     GroupingSet,
     GroupingSetsExpression,
 )
+from snowflake.snowpark._internal.analyzer.select_statement import Selectable
 from snowflake.snowpark._internal.analyzer.snowflake_plan import (
     SnowflakePlan,
     SnowflakePlanBuilder,
@@ -135,6 +136,7 @@ from snowflake.snowpark._internal.analyzer.window_expression import (
     WindowSpecDefinition,
 )
 from snowflake.snowpark._internal.error_message import SnowparkClientExceptionMessages
+from snowflake.snowpark._internal.telemetry import TelemetryField
 from snowflake.snowpark.types import VariantType, _NumericType
 
 ARRAY_BIND_THRESHOLD = 512
@@ -232,6 +234,10 @@ class Analyzer:
             return alias_expression(self.analyze(expr.child), quoted_name)
 
         if isinstance(expr, FunctionExpression):
+            if expr.api_call_source is not None:
+                self.session._conn._telemetry_client.send_function_usage_telemetry(
+                    expr.api_call_source, TelemetryField.FUNC_CAT_USAGE.value
+                )
             return function_expression(
                 expr.name,
                 [self.to_sql_avoid_offset(c) for c in expr.children],
@@ -245,11 +251,19 @@ class Analyzer:
                 return ",".join(list(map(self.analyze, expr.expressions)))
 
         if isinstance(expr, SnowflakeUDF):
+            if expr.api_call_source is not None:
+                self.session._conn._telemetry_client.send_function_usage_telemetry(
+                    expr.api_call_source, TelemetryField.FUNC_CAT_USAGE.value
+                )
             return function_expression(
                 expr.udf_name, list(map(self.analyze, expr.children)), False
             )
 
         if isinstance(expr, TableFunctionExpression):
+            if expr.api_call_source is not None:
+                self.session._conn._telemetry_client.send_function_usage_telemetry(
+                    expr.api_call_source, TelemetryField.FUNC_CAT_USAGE.value
+                )
             return self.table_function_expression_extractor(expr)
 
         if isinstance(expr, TableFunctionPartitionSpecDefinition):
@@ -311,7 +325,7 @@ class Analyzer:
                 expr.sql,
                 self.analyze(expr.expr),
                 expr.offset,
-                self.analyze(expr.default),
+                self.analyze(expr.default) if expr.default else None,
                 expr.ignore_nulls,
             )
 
@@ -415,22 +429,26 @@ class Analyzer:
         for c in logical_plan.children:
             resolved_children[c] = self.resolve(c)
 
-        use_maps = {}
-        # get counts of expr_to_alias keys
-        counts = Counter()
-        for v in resolved_children.values():
-            if v.expr_to_alias:
-                counts.update(list(v.expr_to_alias.keys()))
+        if isinstance(logical_plan, Selectable):
+            # Selectable doesn't have children. It already has the expr_to_alias dict.
+            self.alias_maps_to_use = logical_plan.expr_to_alias
+        else:
+            use_maps = {}
+            # get counts of expr_to_alias keys
+            counts = Counter()
+            for v in resolved_children.values():
+                if v.expr_to_alias:
+                    counts.update(list(v.expr_to_alias.keys()))
 
-        # Keep only non-shared expr_to_alias keys
-        # let (df1.join(df2)).join(df2.join(df3)).select(df2) report error
-        for v in resolved_children.values():
-            if v.expr_to_alias:
-                use_maps.update(
-                    {p: q for p, q in v.expr_to_alias.items() if counts[p] < 2}
-                )
+            # Keep only non-shared expr_to_alias keys
+            # let (df1.join(df2)).join(df2.join(df3)).select(df2) report error
+            for v in resolved_children.values():
+                if v.expr_to_alias:
+                    use_maps.update(
+                        {p: q for p, q in v.expr_to_alias.items() if counts[p] < 2}
+                    )
 
-        self.alias_maps_to_use = use_maps
+            self.alias_maps_to_use = use_maps
         return self.do_resolve_with_resolved_children(logical_plan, resolved_children)
 
     def do_resolve_with_resolved_children(
@@ -571,6 +589,7 @@ class Analyzer:
 
             return self.plan_builder.limit(
                 self.to_sql_avoid_offset(logical_plan.limit_expr),
+                self.to_sql_avoid_offset(logical_plan.offset_expr),
                 resolved_children[logical_plan.child],
                 on_top_of_order_by,
                 logical_plan,
@@ -688,6 +707,7 @@ class Analyzer:
                 if logical_plan.condition
                 else None,
                 resolved_children.get(logical_plan.source_data, None),
+                logical_plan,
             )
 
         if isinstance(logical_plan, TableDelete):
@@ -697,6 +717,7 @@ class Analyzer:
                 if logical_plan.condition
                 else None,
                 resolved_children.get(logical_plan.source_data, None),
+                logical_plan,
             )
 
         if isinstance(logical_plan, TableMerge):
@@ -705,4 +726,8 @@ class Analyzer:
                 resolved_children.get(logical_plan.source),
                 self.analyze(logical_plan.join_expr),
                 [self.analyze(c) for c in logical_plan.clauses],
+                logical_plan,
             )
+
+        if isinstance(logical_plan, Selectable):
+            return self.plan_builder.select_statement(logical_plan)
