@@ -7,6 +7,8 @@ import typing
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 from snowflake.snowpark._internal.analyzer.binary_plan_node import (
+    Except,
+    Intersect,
     JoinType,
     LeftAnti,
     LeftSemi,
@@ -58,6 +60,7 @@ SELECT = " SELECT "
 FROM = " FROM "
 WHERE = " WHERE "
 LIMIT = " LIMIT "
+OFFSET = " OFFSET "
 PIVOT = " PIVOT "
 UNPIVOT = " UNPIVOT "
 FOR = " FOR "
@@ -131,6 +134,10 @@ MATCHED = " MATCHED "
 LISTAGG = " LISTAGG "
 HEADER = " HEADER "
 IGNORE_NULLS = " IGNORE NULLS "
+UNION = " UNION "
+UNION_ALL = " UNION ALL "
+INTERSECT = f" {Intersect.sql} "
+EXCEPT = f" {Except.sql} "
 
 TEMPORARY_STRING_SET = frozenset(["temporary", "temp"])
 
@@ -431,7 +438,6 @@ def range_statement(start: int, end: int, step: int, column_name: str) -> str:
 
 
 def values_statement(output: List[Attribute], data: List[Row]) -> str:
-    table_name = random_name_for_temp_object(TempObjectType.TABLE)
     data_types = [attr.datatype for attr in output]
     names = [quote_name(attr.name) for attr in output]
     rows = []
@@ -441,16 +447,15 @@ def values_statement(output: List[Attribute], data: List[Row]) -> str:
             for value, data_type in zip(row, data_types)
         ]
         rows.append(LEFT_PARENTHESIS + COMMA.join(cells) + RIGHT_PARENTHESIS)
-    query_source = (
-        VALUES
+
+    query = (
+        SELECT
+        + COMMA.join([f"{DOLLAR}{i+1}{AS}{c}" for i, c in enumerate(names)])
+        + FROM
+        + VALUES
         + COMMA.join(rows)
-        + AS
-        + table_name
-        + LEFT_PARENTHESIS
-        + COMMA.join(names)
-        + RIGHT_PARENTHESIS
     )
-    return project_statement([], query_source)
+    return query
 
 
 def empty_values_statement(output: List[Attribute]) -> str:
@@ -622,11 +627,15 @@ def create_table_as_select_statement(
     )
 
 
-def limit_statement(row_count: str, child: str, on_top_of_order_by: bool) -> str:
+def limit_statement(
+    row_count: str, offset: str, child: str, on_top_of_order_by: bool
+) -> str:
     return (
         f"{child if on_top_of_order_by else project_statement([], child)}"
         + LIMIT
         + row_count
+        + OFFSET
+        + offset
     )
 
 
@@ -708,16 +717,24 @@ def file_operation_statement(
     raise ValueError(f"Unsupported file operation type {command}")
 
 
+def convert_value_to_sql_option(value: Optional[Union[str, bool, int, float]]) -> str:
+    if isinstance(value, str):
+        if len(value) > 1 and is_single_quoted(value):
+            return value
+        else:
+            value = value.replace(
+                "'", "''"
+            )  # escape single quotes before adding a pair of quotes
+            return f"'{value}'"
+    else:
+        return str(value)
+
+
 def get_options_statement(options: Dict[str, Any]) -> str:
     return (
         SPACE
         + SPACE.join(
-            # TODO: this next line adds single quotes around the option values, this is
-            #  incorrect to do unless we escape single quotes in the middle of the string
-            # TODO: I can't find documentation that string's repr is guaranteed to be
-            #  surrounded by single quotes, if this ever changes we'll be broken
-            # repr("a") return "'a'" instead of "a". This is what we need for str values. For bool, int, float, repr(v) and str(v) return the same.
-            f"{k}{EQUALS}{v if (isinstance(v, str) and is_single_quoted(v)) else repr(v)}"
+            f"{k}{EQUALS}{convert_value_to_sql_option(v)}"
             for k, v in options.items()
             if v is not None
         )
@@ -791,10 +808,8 @@ def rank_related_function_expression(
         func_name
         + LEFT_PARENTHESIS
         + expr
-        + COMMA
-        + str(offset)
-        + COMMA
-        + default
+        + (COMMA + str(offset) if offset else EMPTY_STRING)
+        + (COMMA + default if default else EMPTY_STRING)
         + RIGHT_PARENTHESIS
         + (IGNORE_NULLS if ignore_nulls else EMPTY_STRING)
     )
@@ -1180,12 +1195,14 @@ def single_quote(value: str) -> str:
         return SINGLE_QUOTE + value + SINGLE_QUOTE
 
 
+ALREADY_QUOTED = re.compile('^(".+")$')
+UNQUOTED_CASE_INSENSITIVE = re.compile("^([_A-Za-z]+[_A-Za-z0-9$]*)$")
+
+
 def quote_name(name: str) -> str:
-    already_quoted = re.compile('^(".+")$')
-    unquoted_case_insensitive = re.compile("^([_A-Za-z]+[_A-Za-z0-9$]*)$")
-    if already_quoted.match(name):
+    if ALREADY_QUOTED.match(name):
         return validate_quoted_name(name)
-    elif unquoted_case_insensitive.match(name):
+    elif UNQUOTED_CASE_INSENSITIVE.match(name):
         return DOUBLE_QUOTE + escape_quotes(name.upper()) + DOUBLE_QUOTE
     else:
         return DOUBLE_QUOTE + escape_quotes(name) + DOUBLE_QUOTE
