@@ -6,7 +6,10 @@ import re
 import sys
 import uuid
 from functools import cached_property, reduce
-from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional, Tuple
+
+if TYPE_CHECKING:
+    from snowflake.snowpark._internal.analyzer.select_statement import Selectable
 
 import snowflake.connector
 import snowflake.snowpark
@@ -57,10 +60,11 @@ from snowflake.snowpark._internal.analyzer.snowflake_plan_node import (
 )
 from snowflake.snowpark._internal.error_message import SnowparkClientExceptionMessages
 from snowflake.snowpark._internal.utils import (
-    COPY_OPTIONS,
     INFER_SCHEMA_FORMAT_TYPES,
     TempObjectType,
     generate_random_alphanumeric,
+    get_copy_into_table_options,
+    is_sql_select_statement,
     random_name_for_temp_object,
 )
 from snowflake.snowpark.row import Row
@@ -566,12 +570,13 @@ class SnowflakePlanBuilder:
     def limit(
         self,
         limit_expr: str,
+        offset_expr: str,
         child: SnowflakePlan,
         on_top_of_oder_by: bool,
         source_plan: Optional[LogicalPlan],
     ) -> SnowflakePlan:
         return self.build(
-            lambda x: limit_statement(limit_expr, x, on_top_of_oder_by),
+            lambda x: limit_statement(limit_expr, offset_expr, x, on_top_of_oder_by),
             child,
             source_plan,
         )
@@ -610,7 +615,7 @@ class SnowflakePlanBuilder:
         if len(child.queries) != 1:
             raise SnowparkClientExceptionMessages.PLAN_CREATE_VIEW_FROM_DDL_DML_OPERATIONS()
 
-        if not child.queries[0].sql.lower().strip().startswith("select"):
+        if not is_sql_select_statement(child.queries[0].sql.lower().strip()):
             raise SnowparkClientExceptionMessages.PLAN_CREATE_VIEWS_FROM_SELECT_ONLY()
 
         return self.build(
@@ -676,16 +681,7 @@ class SnowflakePlanBuilder:
         schema_to_cast: Optional[List[Tuple[str, str]]] = None,
         transformations: Optional[List[str]] = None,
     ):
-        copy_options = {}
-        format_type_options = {}
-
-        for k, v in options.items():
-            if k not in ("PATTERN", "INFER_SCHEMA"):
-                if k in COPY_OPTIONS:
-                    copy_options[k] = v
-                else:
-                    format_type_options[k] = v
-
+        format_type_options, copy_options = get_copy_into_table_options(options)
         pattern = options.get("PATTERN", None)
         # Can only infer the schema for parquet, orc and avro
         infer_schema = (
@@ -751,7 +747,7 @@ class SnowflakePlanBuilder:
                 None,
             )
         else:  # otherwise use COPY
-            if "FORCE" in copy_options and copy_options["FORCE"].lower() != "true":
+            if "FORCE" in copy_options and str(copy_options["FORCE"]).lower() != "true":
                 raise SnowparkClientExceptionMessages.PLAN_COPY_DONT_SUPPORT_SKIP_LOADED_FILES(
                     copy_options["FORCE"]
                 )
@@ -919,42 +915,69 @@ class SnowflakePlanBuilder:
         assignments: Dict[str, str],
         condition: Optional[str],
         source_data: Optional[SnowflakePlan],
+        source_plan: Optional[LogicalPlan],
     ) -> SnowflakePlan:
-        return self.query(
-            update_statement(
-                table_name,
-                assignments,
-                condition,
-                source_data.queries[-1].sql
-                if source_data and source_data.queries
-                else None,
-            ),
-            None,
-        )
+        if source_data:
+            return self.build(
+                lambda x: update_statement(
+                    table_name,
+                    assignments,
+                    condition,
+                    x,
+                ),
+                source_data,
+                source_plan,
+            )
+        else:
+            return self.query(
+                update_statement(
+                    table_name,
+                    assignments,
+                    condition,
+                    None,
+                ),
+                source_plan,
+            )
 
     def delete(
         self,
         table_name: str,
         condition: Optional[str],
         source_data: Optional[SnowflakePlan],
+        source_plan: Optional[LogicalPlan],
     ) -> SnowflakePlan:
-        return self.query(
-            delete_statement(
-                table_name,
-                condition,
-                source_data.queries[-1].sql
-                if source_data and source_data.queries
-                else None,
-            ),
-            None,
-        )
+        if source_data:
+            return self.build(
+                lambda x: delete_statement(
+                    table_name,
+                    condition,
+                    x,
+                ),
+                source_data,
+                source_plan,
+            )
+        else:
+            return self.query(
+                delete_statement(
+                    table_name,
+                    condition,
+                    None,
+                ),
+                source_plan,
+            )
 
     def merge(
-        self, table_name: str, source: SnowflakePlan, join_expr: str, clauses: List[str]
+        self,
+        table_name: str,
+        source_data: SnowflakePlan,
+        join_expr: str,
+        clauses: List[str],
+        source_plan: Optional[LogicalPlan],
     ) -> SnowflakePlan:
-        return self.query(
-            merge_statement(table_name, source.queries[-1].sql, join_expr, clauses),
-            None,
+        return self.build(
+            lambda x: merge_statement(table_name, x, join_expr, clauses),
+            source_data,
+            source_plan,
         )
 
     def lateral(
@@ -981,10 +1004,13 @@ class SnowflakePlanBuilder:
             source_plan,
         )
 
+    def select_statement(self, selectable: "Selectable") -> SnowflakePlan:
+        return selectable.snowflake_plan
+
     def add_result_scan_if_not_select(self, plan: SnowflakePlan) -> SnowflakePlan:
         if isinstance(plan.source_plan, SetOperation):
             return plan
-        elif plan.queries[-1].sql.strip().lower().startswith("select"):
+        elif is_sql_select_statement(plan.queries[-1].sql):
             return plan
         else:
             new_queries = plan.queries + [
