@@ -1,9 +1,12 @@
 #
 # Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
 #
-from typing import List
+from typing import List, Optional, Set
 
-from snowflake.snowpark._internal.analyzer.expression import Expression
+from snowflake.snowpark._internal.analyzer.expression import (
+    Expression,
+    derive_dependent_columns,
+)
 from snowflake.snowpark._internal.analyzer.sort_expression import SortOrder
 
 
@@ -56,6 +59,9 @@ class SpecifiedWindowFrame(WindowFrame):
         self.lower = lower
         self.upper = upper
 
+    def dependent_column_names(self) -> Optional[Set[str]]:
+        return derive_dependent_columns(self.lower, self.upper)
+
 
 class WindowSpecDefinition(Expression):
     def __init__(
@@ -69,6 +75,11 @@ class WindowSpecDefinition(Expression):
         self.order_spec = order_spec
         self.frame_spec = frame_spec
 
+    def dependent_column_names(self) -> Optional[Set[str]]:
+        return derive_dependent_columns(
+            *self.partition_spec, *self.order_spec, self.frame_spec
+        )
+
 
 class WindowExpression(Expression):
     def __init__(
@@ -78,6 +89,9 @@ class WindowExpression(Expression):
         self.window_function = window_function
         self.window_spec = window_spec
 
+    def dependent_column_names(self) -> Optional[Set[str]]:
+        return derive_dependent_columns(self.window_function, self.window_spec)
+
 
 class RankRelatedFunctionExpression(Expression):
     sql: str
@@ -91,6 +105,9 @@ class RankRelatedFunctionExpression(Expression):
         self.default = default
         self.ignore_nulls = ignore_nulls
 
+    def dependent_column_names(self) -> Optional[Set[str]]:
+        return derive_dependent_columns(self.expr, self.default)
+
 
 class Lag(RankRelatedFunctionExpression):
     sql = "LAG"
@@ -98,3 +115,11 @@ class Lag(RankRelatedFunctionExpression):
 
 class Lead(RankRelatedFunctionExpression):
     sql = "LEAD"
+
+
+class LastValue(RankRelatedFunctionExpression):
+    sql = "LAST_VALUE"
+
+
+class FirstValue(RankRelatedFunctionExpression):
+    sql = "FIRST_VALUE"
