@@ -20,7 +20,18 @@ import zipfile
 from enum import Enum
 from json import JSONEncoder
 from random import choice
-from typing import IO, Any, Callable, Dict, Iterator, List, Literal, Optional, Type
+from typing import (
+    IO,
+    Any,
+    Callable,
+    Dict,
+    Iterator,
+    List,
+    Literal,
+    Optional,
+    Tuple,
+    Type,
+)
 
 import snowflake.snowpark
 from snowflake.connector.cursor import ResultMetadata, SnowflakeCursor
@@ -42,6 +53,8 @@ SNOWFLAKE_QUOTED_ID_PATTERN = '("([^"]|""){1,255}")'
 SNOWFLAKE_ID_PATTERN = (
     f"({SNOWFLAKE_UNQUOTED_ID_PATTERN}|{SNOWFLAKE_QUOTED_ID_PATTERN})"
 )
+SNOWFLAKE_CASE_INSENSITIVE_QUOTED_ID_PATTERN = r'("([A-Z_][A-Z0-9_\$]{0,255})")'
+SNOWFLAKE_CASE_INSENSITIVE_UNQUOTED_SUFFIX_PATTERN = r"([a-zA-Z0-9_\$]{0,255})"
 
 # Valid name can be:
 #   identifier
@@ -52,6 +65,13 @@ SNOWFLAKE_OBJECT_RE_PATTERN = re.compile(
     f"^(({SNOWFLAKE_ID_PATTERN}\\.){{0,2}}|({SNOWFLAKE_ID_PATTERN}\\.\\.)){SNOWFLAKE_ID_PATTERN}$"
 )
 
+SNOWFLAKE_CASE_INSENSITIVE_QUOTED_ID_RE_PATTERN = re.compile(
+    SNOWFLAKE_CASE_INSENSITIVE_QUOTED_ID_PATTERN
+)
+SNOWFLAKE_CASE_INSENSITIVE_UNQUOTED_SUFFIX_RE_PATTERN = re.compile(
+    SNOWFLAKE_CASE_INSENSITIVE_UNQUOTED_SUFFIX_PATTERN
+)
+
 # "%?" is for table stage
 SNOWFLAKE_STAGE_NAME_PATTERN = f"(%?{SNOWFLAKE_ID_PATTERN})"
 
@@ -59,6 +79,10 @@ SNOWFLAKE_STAGE_NAME_PATTERN = f"(%?{SNOWFLAKE_ID_PATTERN})"
 TEMP_OBJECT_NAME_PREFIX = "SNOWPARK_TEMP_"
 ALPHANUMERIC = string.digits + string.ascii_lowercase
 
+# select and CTE (https://docs.snowflake.com/en/sql-reference/constructs/with.html) are select statements in Snowflake.
+SNOWFLAKE_SELECT_SQL_PREFIX_PATTERN = re.compile(
+    r"^(\s|\()*(select|with)", re.IGNORECASE
+)
 
 # A set of widely-used packages,
 # whose names in pypi are different from their package name
@@ -111,6 +135,20 @@ COPY_OPTIONS = {
     "LOAD_UNCERTAIN_FILES",
 }
 
+NON_FORMAT_TYPE_OPTIONS = {
+    "PATTERN",
+    "VALIDATION_MODE",
+    "FILE_FORMAT",
+    "FORMAT_NAME",
+    "FILES",
+    # The following are not copy into SQL command options but client side options.
+    "INFER_SCHEMA",
+    "FORMAT_TYPE_OPTIONS",
+    "TARGET_COLUMNS",
+    "TRANSFORMATIONS",
+    "COPY_OPTIONS",
+}
+
 QUERY_TAG_STRING = "QUERY_TAG"
 SKIP_LEVELS_TWO = (
     2  # limit traceback to return up to 2 stack trace entries from traceback object tb
@@ -165,6 +203,17 @@ def is_single_quoted(name: str) -> bool:
     return name.startswith("'") and name.endswith("'")
 
 
+def is_snowflake_quoted_id_case_insensitive(name: str) -> bool:
+    return SNOWFLAKE_CASE_INSENSITIVE_QUOTED_ID_RE_PATTERN.fullmatch(name) is not None
+
+
+def is_snowflake_unquoted_suffix_case_insensitive(name: str) -> bool:
+    return (
+        SNOWFLAKE_CASE_INSENSITIVE_UNQUOTED_SUFFIX_RE_PATTERN.fullmatch(name)
+        is not None
+    )
+
+
 def unwrap_single_quote(name: str) -> str:
     new_name = name.strip()
     if is_single_quoted(new_name):
@@ -173,6 +222,10 @@ def unwrap_single_quote(name: str) -> str:
     return new_name
 
 
+def is_sql_select_statement(sql: str) -> bool:
+    return SNOWFLAKE_SELECT_SQL_PREFIX_PATTERN.match(sql) is not None
+
+
 def normalize_path(path: str, is_local: bool) -> str:
     """
     Get a normalized path of a local file or remote stage location for PUT/GET commands.
@@ -295,7 +348,7 @@ def zip_file_or_directory_to_stream(
     input_stream.close()
 
 
-def parse_positional_args_to_list(*inputs) -> List:
+def parse_positional_args_to_list(*inputs: Any) -> List:
     """Convert the positional arguments to a list."""
     if len(inputs) == 1:
         return (
@@ -526,7 +579,7 @@ def warning(name: str, text: str, warning_times: int = 1) -> None:
 
 
 def func_decorator(
-    decorator_type: Literal["deprecated", "experimental"],
+    decorator_type: Literal["deprecated", "experimental", "in private preview"],
     *,
     version: str,
     extra_warning_text: str,
@@ -535,7 +588,7 @@ def func_decorator(
     def wrapper(func):
         warning_text = (
             f"{func.__qualname__}() is {decorator_type} since {version}. "
-            f"{'Do not use it in production. ' if decorator_type == 'experimental' else ''}"
+            f"{'Do not use it in production. ' if decorator_type in ('experimental', 'in private preview') else ''}"
             f"{extra_warning_text}"
         )
         doc_string_text = f"This function or method is {decorator_type} since {version}. {extra_doc_string} \n\n"
@@ -573,6 +626,17 @@ def experimental(
     )
 
 
+def private_preview(
+    *, version: str, extra_warning_text: str = "", extra_doc_string: str = ""
+) -> Callable:
+    return func_decorator(
+        "in private preview",
+        version=version,
+        extra_warning_text=extra_warning_text,
+        extra_doc_string=extra_doc_string,
+    )
+
+
 def get_temp_type_for_object(use_scoped_temp_objects: bool, is_generated: bool) -> str:
     return (
         SCOPED_TEMPORARY_STRING
@@ -590,3 +654,16 @@ def check_is_pandas_dataframe_in_to_pandas(result: Any) -> None:
             "list of Row objects for a non-SELECT statement, then convert it to a "
             "Pandas DataFrame."
         )
+
+
+def get_copy_into_table_options(
+    options: Dict[str, Any]
+) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+    file_format_type_options = options.get("FORMAT_TYPE_OPTIONS", {})
+    copy_options = options.get("COPY_OPTIONS", {})
+    for k, v in options.items():
+        if k in COPY_OPTIONS:
+            copy_options[k] = v
+        elif k not in NON_FORMAT_TYPE_OPTIONS:
+            file_format_type_options[k] = v
+    return file_format_type_options, copy_options
