@@ -12,13 +12,17 @@ from snowflake.snowpark._internal.analyzer.analyzer_utils import (
     quote_name_without_upper_casing,
 )
 from snowflake.snowpark._internal.analyzer.expression import Attribute
+from snowflake.snowpark._internal.analyzer.select_statement import (
+    SelectSnowflakePlan,
+    SelectStatement,
+)
 from snowflake.snowpark._internal.error_message import SnowparkClientExceptionMessages
 from snowflake.snowpark._internal.telemetry import set_api_call_source
 from snowflake.snowpark._internal.type_utils import convert_sf_to_sp_type
 from snowflake.snowpark._internal.utils import (
-    COPY_OPTIONS,
     INFER_SCHEMA_FORMAT_TYPES,
     TempObjectType,
+    get_copy_into_table_options,
     random_name_for_temp_object,
 )
 from snowflake.snowpark.dataframe import DataFrame
@@ -110,7 +114,7 @@ class DataFrameReader:
             >>> csv_df.collect()
             [Row(A=1, B='one', C=1.2), Row(A=2, B='two', C=2.2), Row(A=3, B='three', C=3.3), Row(A=4, B='four', C=4.4)]
 
-    To load Parquet, ORC and AVRO files, no schema is accepted becasue the schema will be automatically inferred.
+    To load Parquet, ORC and AVRO files, no schema is accepted because the schema will be automatically inferred.
     Inferring the schema can be disabled by setting option "infer_schema" to ``False``. Then you can use ``$1`` to access
     the column data as an OBJECT.
 
@@ -273,16 +277,35 @@ class DataFrameReader:
 
         self._file_path = path
         self._file_type = "csv"
-        df = DataFrame(
-            self._session,
-            self._session._plan_builder.read_file(
-                path,
-                self._file_type,
-                self._cur_options,
-                self._session.get_fully_qualified_current_schema(),
-                self._user_schema._to_attributes(),
-            ),
-        )
+
+        if self._session.sql_simplifier_enabled:
+            df = DataFrame(
+                self._session,
+                SelectStatement(
+                    from_=SelectSnowflakePlan(
+                        self._session._plan_builder.read_file(
+                            path,
+                            self._file_type,
+                            self._cur_options,
+                            self._session.get_fully_qualified_current_schema(),
+                            self._user_schema._to_attributes(),
+                        ),
+                        analyzer=self._session._analyzer,
+                    ),
+                    analyzer=self._session._analyzer,
+                ),
+            )
+        else:
+            df = DataFrame(
+                self._session,
+                self._session._plan_builder.read_file(
+                    path,
+                    self._file_type,
+                    self._cur_options,
+                    self._session.get_fully_qualified_current_schema(),
+                    self._user_schema._to_attributes(),
+                ),
+            )
         df._reader = self
         set_api_call_source(df, "DataFrameReader.csv")
         return df
@@ -378,10 +401,7 @@ class DataFrameReader:
         self._file_path = path
         self._file_type = format
 
-        format_type_options = {}
-        for k, v in self._cur_options.items():
-            if k not in ("PATTERN", "INFER_SCHEMA") and k not in COPY_OPTIONS:
-                format_type_options[k] = v
+        format_type_options, _ = get_copy_into_table_options(self._cur_options)
 
         self._infer_schema = (
             self._cur_options.get("INFER_SCHEMA", True)
@@ -463,18 +483,38 @@ class DataFrameReader:
                         drop_tmp_file_format_if_exists_query, is_ddl_on_temp_object=True
                     )
 
-        df = DataFrame(
-            self._session,
-            self._session._plan_builder.read_file(
-                path,
-                format,
-                self._cur_options,
-                self._session.get_fully_qualified_current_schema(),
-                schema,
-                schema_to_cast=schema_to_cast,
-                transformations=read_file_transformations,
-            ),
-        )
+        if self._session.sql_simplifier_enabled:
+            df = DataFrame(
+                self._session,
+                SelectStatement(
+                    from_=SelectSnowflakePlan(
+                        self._session._plan_builder.read_file(
+                            path,
+                            format,
+                            self._cur_options,
+                            self._session.get_fully_qualified_current_schema(),
+                            schema,
+                            schema_to_cast=schema_to_cast,
+                            transformations=read_file_transformations,
+                        ),
+                        analyzer=self._session._analyzer,
+                    ),
+                    analyzer=self._session._analyzer,
+                ),
+            )
+        else:
+            df = DataFrame(
+                self._session,
+                self._session._plan_builder.read_file(
+                    path,
+                    format,
+                    self._cur_options,
+                    self._session.get_fully_qualified_current_schema(),
+                    schema,
+                    schema_to_cast=schema_to_cast,
+                    transformations=read_file_transformations,
+                ),
+            )
         df._reader = self
         set_api_call_source(df, f"DataFrameReader.{format.lower()}")
         return df
