@@ -27,6 +27,12 @@ from snowflake.snowpark._internal.analyzer.analyzer_utils import (
 )
 from snowflake.snowpark._internal.analyzer.datatype_mapper import str_to_sql, to_sql
 from snowflake.snowpark._internal.analyzer.expression import Attribute
+from snowflake.snowpark._internal.analyzer.select_statement import (
+    SelectSnowflakePlan,
+    SelectSQL,
+    SelectStatement,
+    SelectTableFunction,
+)
 from snowflake.snowpark._internal.analyzer.snowflake_plan import SnowflakePlanBuilder
 from snowflake.snowpark._internal.analyzer.snowflake_plan_node import (
     Range,
@@ -38,7 +44,7 @@ from snowflake.snowpark._internal.analyzer.table_function import (
 )
 from snowflake.snowpark._internal.error_message import SnowparkClientExceptionMessages
 from snowflake.snowpark._internal.server_connection import ServerConnection
-from snowflake.snowpark._internal.telemetry import TelemetryField, set_api_call_source
+from snowflake.snowpark._internal.telemetry import set_api_call_source
 from snowflake.snowpark._internal.type_utils import (
     ColumnOrName,
     infer_schema,
@@ -53,6 +59,7 @@ from snowflake.snowpark._internal.utils import (
     TempObjectType,
     calculate_checksum,
     deprecated,
+    experimental,
     get_connector_version,
     get_os_name,
     get_python_version,
@@ -69,12 +76,14 @@ from snowflake.snowpark._internal.utils import (
     warning,
     zip_file_or_directory_to_stream,
 )
+from snowflake.snowpark.async_job import AsyncJob
 from snowflake.snowpark.column import Column
 from snowflake.snowpark.context import _use_scoped_temp_objects
 from snowflake.snowpark.dataframe import DataFrame
 from snowflake.snowpark.dataframe_reader import DataFrameReader
 from snowflake.snowpark.file_operation import FileOperation
 from snowflake.snowpark.functions import (
+    array_agg,
     col,
     column,
     parse_json,
@@ -268,6 +277,7 @@ class Session:
         self._file = FileOperation(self)
 
         self._analyzer = Analyzer(self)
+        self._sql_simplifier_enabled: bool = False
         _logger.info("Snowpark Session information: %s", self._session_info)
 
     def __enter__(self):
@@ -309,6 +319,19 @@ class Session:
             finally:
                 _remove_session(self)
 
+    @property
+    def sql_simplifier_enabled(self) -> bool:
+        return self._sql_simplifier_enabled
+
+    @sql_simplifier_enabled.setter
+    def sql_simplifier_enabled(self, value: bool) -> None:
+        """Set to ``True`` to use the SQL simplifier.
+        The generated SQLs from ``DataFrame`` transformations would have fewer layers of nested queries if the SQL simplifier is enabled."""
+        self._conn._telemetry_client.send_sql_simplifier_telemetry(
+            self._session_id, value
+        )
+        self._sql_simplifier_enabled = value
+
     def cancel_all(self) -> None:
         """
         Cancel all action methods that are running currently.
@@ -719,28 +742,13 @@ class Session:
         validate_package: bool = True,
         include_pandas: bool = False,
     ) -> List[str]:
-        valid_packages = (
-            {
-                p[0]: p[1]
-                for p in self._run_query(
-                    "select package_name, version from information_schema.packages where language='python'"
-                )
-            }
-            if validate_package
-            else None
-        )
-
-        result_dict = (
-            existing_packages_dict if existing_packages_dict is not None else {}
-        )
+        package_dict = dict()
         for package in packages:
             if isinstance(package, ModuleType):
-                package = MODULE_NAME_TO_PACKAGE_NAME_MAP.get(
+                package_name = MODULE_NAME_TO_PACKAGE_NAME_MAP.get(
                     package.__name__, package.__name__
                 )
-                package = (
-                    f"{package}=={pkg_resources.get_distribution(package).version}"
-                )
+                package = f"{package_name}=={pkg_resources.get_distribution(package_name).version}"
                 use_local_version = True
             else:
                 package = package.strip().lower()
@@ -752,34 +760,68 @@ class Session:
             package_name = (
                 package if not use_local_version and "_" in package else package_req.key
             )
+            package_dict[package] = (package_name, use_local_version, package_req)
+
+        valid_packages = (
+            {
+                p[0]: json.loads(p[1])
+                for p in self.table("information_schema.packages")
+                .filter(
+                    (col("language") == "python")
+                    & (col("package_name").in_([v[0] for v in package_dict.values()]))
+                )
+                .group_by("package_name")
+                .agg(array_agg("version"))
+                ._internal_collect_with_tag()
+            }
+            if validate_package and package_dict
+            else None
+        )
+
+        result_dict = (
+            existing_packages_dict if existing_packages_dict is not None else {}
+        )
+        for package, package_info in package_dict.items():
+            package_name, use_local_version, package_req = package_info
+            package_version_req = package_req.specs[0][1] if package_req.specs else None
+
             if validate_package:
+                unavailable_pkg_err_msg = (
+                    "it is not available in Snowflake. Check information_schema.packages "
+                    "to see available packages for UDFs. If this package is a "
+                    '"pure-Python" package, you can find the directory of this package '
+                    "and add it via session.add_import(). See details at "
+                    "https://docs.snowflake.com/en/developer-guide/snowpark/python/creating-udfs.html#using-third-party-packages-from-anaconda-in-a-udf."
+                )
+
                 if package_name not in valid_packages:
                     is_anaconda_terms_acknowledged = self._run_query(
                         "select system$are_anaconda_terms_acknowledged()"
                     )[0][0]
                     if is_anaconda_terms_acknowledged:
-                        detailed_err_msg = (
-                            "it is not available in Snowflake. Check information_schema.packages "
-                            "to see available packages for UDFs. If this package is a "
-                            '"pure-Python" package, you can find the directory of this package '
-                            "and add it via session.add_import(). See details at "
-                            "https://docs.snowflake.com/en/developer-guide/snowpark/python/creating-udfs.html#using-third-party-packages-from-anaconda-in-a-udf."
-                        )
+                        detailed_err_msg = unavailable_pkg_err_msg
                     else:
                         detailed_err_msg = (
                             "Anaconda terms must be accepted by ORGADMIN to use "
                             "Anaconda 3rd party packages. Please follow the instructions at "
                             "https://docs.snowflake.com/en/developer-guide/udf/python/udf-python-packages.html#using-third-party-packages-from-anaconda."
                         )
+
                     raise ValueError(
                         f"Cannot add package {package_name} because {detailed_err_msg}"
                     )
+                elif package_version_req and not any(
+                    v in package_req for v in valid_packages[package_name]
+                ):
+                    raise ValueError(
+                        f"Cannot add package {package_name}=={package_version_req} because {unavailable_pkg_err_msg}"
+                    )
                 elif not use_local_version:
                     try:
                         package_client_version = pkg_resources.get_distribution(
                             package_name
                         ).version
-                        if package_client_version not in package_req:
+                        if package_client_version not in valid_packages[package_name]:
                             _logger.warning(
                                 "The version of package %s in the local environment is %s, "
                                 "which does not fit the criteria for the requirement %s. "
@@ -863,6 +905,11 @@ class Session:
             name: A string or list of strings that specify the table name or
                 fully-qualified object identifier (database name, schema name, and table name).
 
+            Note:
+                If your table name contains special characters, use double quotes to mark it like this, ``session.table('"my table"')``.
+                For fully qualified names, you need to use double quotes separately like this, ``session.table('"my db"."my schema"."my.table"')``.
+                Refer to `Identifier Requirements <https://docs.snowflake.com/en/sql-reference/identifiers-syntax.html>`_.
+
         Examples::
 
             >>> df1 = session.create_dataframe([[1, 2], [3, 4]], schema=["a", "b"])
@@ -935,10 +982,20 @@ class Session:
         func_expr = _create_table_function_expression(
             func_name, *func_arguments, **func_named_arguments
         )
-        d = DataFrame(
-            self,
-            TableFunctionRelation(func_expr),
-        )
+
+        if self.sql_simplifier_enabled:
+            d = DataFrame(
+                self,
+                SelectStatement(
+                    from_=SelectTableFunction(func_expr, analyzer=self._analyzer),
+                    analyzer=self._analyzer,
+                ),
+            )
+        else:
+            d = DataFrame(
+                self,
+                TableFunctionRelation(func_expr),
+            )
         set_api_call_source(d, "Session.table_function")
         return d
 
@@ -959,12 +1016,22 @@ class Session:
             >>> df.collect()
             [Row(1/2=Decimal('0.500000'))]
         """
-        return DataFrame(
-            self,
-            self._plan_builder.query(
-                query, None, api_calls=[{TelemetryField.NAME.value: "Session.sql"}]
-            ),
-        )
+
+        if self.sql_simplifier_enabled:
+            d = DataFrame(
+                self,
+                SelectStatement(
+                    from_=SelectSQL(query, analyzer=self._analyzer),
+                    analyzer=self._analyzer,
+                ),
+            )
+        else:
+            d = DataFrame(
+                self,
+                self._plan_builder.query(query, None),
+            )
+        set_api_call_source(d, "Session.sql")
+        return d
 
     @property
     def read(self) -> "DataFrameReader":
@@ -1044,7 +1111,7 @@ class Session:
                 that to avoid breaking changes, currently when this is set to True, it overrides ``table_type``.
             overwrite: Default value is ``False`` and the Pandas DataFrame data is appended to the existing table. If set to ``True`` and if auto_create_table is also set to ``True``,
                 then it drops the table. If set to ``True`` and if auto_create_table is set to ``False``,
-                then it trunctates the table. Note that in both cases (when overwrite is set to ``True``) it will replace the existing
+                then it truncates the table. Note that in both cases (when overwrite is set to ``True``) it will replace the existing
                 contents of the table with that of the passed in Pandas DataFrame.
             table_type: The table type of table to be created. The supported values are: ``temp``, ``temporary``,
                         and ``transient``. An empty string means to create a permanent table. Learn more about table
@@ -1378,8 +1445,20 @@ class Session:
             else:
                 project_columns.append(column(name))
 
-        df = DataFrame(self, SnowflakeValues(attrs, converted)).select(project_columns)
-        # Get rid of the select statement api call here
+        if self.sql_simplifier_enabled:
+            df = DataFrame(
+                self,
+                SelectStatement(
+                    from_=SelectSnowflakePlan(
+                        SnowflakeValues(attrs, converted), analyzer=self._analyzer
+                    ),
+                    analyzer=self._analyzer,
+                ),
+            ).select(project_columns)
+        else:
+            df = DataFrame(self, SnowflakeValues(attrs, converted)).select(
+                project_columns
+            )
         set_api_call_source(df, "Session.create_dataframe[values]")
         return df
 
@@ -1405,10 +1484,34 @@ class Session:
             [Row(ID=1), Row(ID=3), Row(ID=5), Row(ID=7), Row(ID=9)]
         """
         range_plan = Range(0, start, step) if end is None else Range(start, end, step)
-        df = DataFrame(self, range_plan)
+
+        if self.sql_simplifier_enabled:
+            df = DataFrame(
+                self,
+                SelectStatement(
+                    from_=SelectSnowflakePlan(range_plan, analyzer=self._analyzer),
+                    analyzer=self._analyzer,
+                ),
+            )
+        else:
+            df = DataFrame(self, range_plan)
         set_api_call_source(df, "Session.range")
         return df
 
+    @experimental(version="0.12.0")
+    def create_async_job(self, query_id: str) -> AsyncJob:
+        """
+        Creates an :class:`AsyncJob` from a query ID.
+
+        See also:
+            :class:`AsyncJob`
+        """
+        if is_in_stored_procedure():
+            raise NotImplementedError(
+                "Async query is not supported in stored procedure yet"
+            )
+        return AsyncJob(query_id, None, self)
+
     def get_current_account(self) -> Optional[str]:
         """
         Returns the name of the current account for the Python connector session attached
@@ -1485,6 +1588,21 @@ class Session:
         """
         self._use_object(role, "role")
 
+    def use_secondary_roles(self, roles: Optional[Literal["all", "none"]]) -> None:
+        """
+        Specifies the active/current secondary roles for the session.
+        The currently-active secondary roles set the context that determines whether
+        the current user has the necessary privileges to perform SQL actions.
+
+        Args:
+            roles: "all" or "none". ``None`` means "none".
+
+        References: `Snowflake command USE SECONDARY ROLES <https://docs.snowflake.com/en/sql-reference/sql/use-secondary-roles.html>`_.
+        """
+        self._run_query(
+            f"use secondary roles {'none' if roles is None else roles.lower()}"
+        )
+
     def _use_object(self, object_name: str, object_type: str) -> None:
         if object_name:
             validate_object_name(object_name)
