@@ -11,6 +11,7 @@ from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 import snowflake.snowpark
 from snowflake.connector import ProgrammingError
 from snowflake.snowpark._internal.error_message import SnowparkClientExceptionMessages
+from snowflake.snowpark._internal.telemetry import TelemetryField
 from snowflake.snowpark._internal.type_utils import convert_sp_to_sf_type
 from snowflake.snowpark._internal.udf_utils import (
     UDFColumn,
@@ -64,11 +65,25 @@ class StoredProcedure:
         *args: Any,
         session: Optional["snowflake.snowpark.session.Session"] = None,
     ) -> any:
-        session = session or snowflake.snowpark.session._get_active_session()
+        if args and isinstance(args[0], snowflake.snowpark.session.Session):
+            if session:
+                raise ValueError(
+                    "Two sessions specified in arguments. Session should either be the first argument or as "
+                    "a named argument at the end, but not both"
+                )
+            session = args[0]
+            args = args[1:]
+        else:
+            session = session or snowflake.snowpark.session._get_active_session()
+
         if len(self._input_types) != len(args):
             raise ValueError(
                 f"Incorrect number of arguments passed to the stored procedure. Expected: {len(self._input_types)}, Found: {len(args)}"
             )
+
+        session._conn._telemetry_client.send_function_usage_telemetry(
+            "StoredProcedure.__call__", TelemetryField.FUNC_CAT_USAGE.value
+        )
         return session.call(self.name, *args)
 
 
@@ -407,6 +422,7 @@ class StoredProcedureRegistration:
             parallel,
             statement_params=statement_params,
             execute_as=execute_as,
+            api_call_source="StoredProcedureRegistration.register",
         )
 
     def register_from_file(
@@ -508,6 +524,7 @@ class StoredProcedureRegistration:
             replace,
             parallel,
             statement_params=statement_params,
+            api_call_source="StoredProcedureRegistration.register_from_file",
         )
 
     def _do_register_sp(
@@ -524,6 +541,7 @@ class StoredProcedureRegistration:
         *,
         statement_params: Optional[Dict[str, str]] = None,
         execute_as: typing.Literal["caller", "owner"] = "owner",
+        api_call_source: str,
     ) -> StoredProcedure:
         (
             udf_name,
@@ -582,6 +600,7 @@ class StoredProcedureRegistration:
                 replace=replace,
                 inline_python_code=code,
                 execute_as=execute_as,
+                api_call_source=api_call_source,
             )
         # an exception might happen during registering a stored procedure
         # (e.g., a dependency might not be found on the stage),
