@@ -2,10 +2,14 @@
 #
 # Copyright (c) 2012-2022 Snowflake Computing Inc. All rights reserved.
 #
-from typing import Dict, Iterable, List, NamedTuple, Optional, Union
+from typing import Dict, Iterable, List, NamedTuple, Optional, Union, overload
 
 import snowflake.snowpark
 from snowflake.snowpark._internal.analyzer.binary_plan_node import create_join_type
+from snowflake.snowpark._internal.analyzer.select_statement import (
+    SelectableEntity,
+    SelectStatement,
+)
 from snowflake.snowpark._internal.analyzer.snowflake_plan_node import UnresolvedRelation
 from snowflake.snowpark._internal.analyzer.table_merge_expression import (
     DeleteMergeExpression,
@@ -256,6 +260,12 @@ class Table(DataFrame):
             session, session._analyzer.resolve(UnresolvedRelation(table_name))
         )
         self.table_name: str = table_name  #: The table name
+
+        if self._session.sql_simplifier_enabled:
+            self._select_statement = SelectStatement(
+                from_=SelectableEntity(table_name, analyzer=session._analyzer),
+                analyzer=session._analyzer,
+            )
         # By default, the set the initial API call to say 'Table.__init__' since
         # people could instantiate a table directly. This value is overwritten when
         # created from Session object
@@ -264,6 +274,12 @@ class Table(DataFrame):
     def __copy__(self) -> "Table":
         return Table(self.table_name, self._session)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.drop_table()
+
     def sample(
         self,
         frac: Optional[float] = None,
@@ -309,11 +325,35 @@ class Table(DataFrame):
 
         # The analyzer will generate a sql with subquery. So we build the sql directly without using the analyzer.
         sampling_method_text = sampling_method or ""
-        frac_or_rowcount_text = str(frac * 100.0) if frac is not None else f"{n} rows"
-        seed_text = f" seed ({seed})" if seed is not None else ""
-        sql_text = f"select * from {self.table_name} sample {sampling_method_text} ({frac_or_rowcount_text}) {seed_text}"
+        frac_or_rowcount_text = str(frac * 100.0) if frac is not None else f"{n} ROWS"
+        seed_text = f" SEED ({seed})" if seed is not None else ""
+        sql_text = f"SELECT * FROM {self.table_name} SAMPLE {sampling_method_text} ({frac_or_rowcount_text}) {seed_text}"
         return self._session.sql(sql_text)
 
+    @overload
+    def update(
+        self,
+        assignments: Dict[str, ColumnOrLiteral],
+        condition: Optional[Column] = None,
+        source: Optional[DataFrame] = None,
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+        block: bool = True,
+    ) -> UpdateResult:
+        ...
+
+    @overload
+    def update(
+        self,
+        assignments: Dict[str, ColumnOrLiteral],
+        condition: Optional[Column] = None,
+        source: Optional[DataFrame] = None,
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+        block: bool = False,
+    ) -> "snowflake.snowpark.AsyncJob":
+        ...
+
     def update(
         self,
         assignments: Dict[str, ColumnOrLiteral],
@@ -398,10 +438,32 @@ class Table(DataFrame):
         result = new_df._internal_collect_with_tag(
             statement_params=statement_params,
             block=block,
-            data_type=snowflake.snowpark.async_job._AsyncDataType.UPDATE,
+            data_type=snowflake.snowpark.async_job._AsyncResultType.UPDATE,
         )
         return _get_update_result(result) if block else result
 
+    @overload
+    def delete(
+        self,
+        condition: Optional[Column] = None,
+        source: Optional[DataFrame] = None,
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+        block: bool = True,
+    ) -> DeleteResult:
+        ...
+
+    @overload
+    def delete(
+        self,
+        condition: Optional[Column] = None,
+        source: Optional[DataFrame] = None,
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+        block: bool = False,
+    ) -> "snowflake.snowpark.AsyncJob":
+        ...
+
     def delete(
         self,
         condition: Optional[Column] = None,
@@ -476,10 +538,34 @@ class Table(DataFrame):
         result = new_df._internal_collect_with_tag(
             statement_params=statement_params,
             block=block,
-            data_type=snowflake.snowpark.async_job._AsyncDataType.DELETE,
+            data_type=snowflake.snowpark.async_job._AsyncResultType.DELETE,
         )
         return _get_delete_result(result) if block else result
 
+    @overload
+    def merge(
+        self,
+        source: DataFrame,
+        join_expr: Column,
+        clauses: Iterable[Union[WhenMatchedClause, WhenNotMatchedClause]],
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+        block: bool = True,
+    ) -> MergeResult:
+        ...
+
+    @overload
+    def merge(
+        self,
+        source: DataFrame,
+        join_expr: Column,
+        clauses: Iterable[Union[WhenMatchedClause, WhenNotMatchedClause]],
+        *,
+        statement_params: Optional[Dict[str, str]] = None,
+        block: bool = False,
+    ) -> "snowflake.snowpark.AsyncJob":
+        ...
+
     def merge(
         self,
         source: DataFrame,
@@ -559,7 +645,7 @@ class Table(DataFrame):
         result = new_df._internal_collect_with_tag(
             statement_params=statement_params,
             block=block,
-            data_type=snowflake.snowpark.async_job._AsyncDataType.MERGE,
+            data_type=snowflake.snowpark.async_job._AsyncResultType.MERGE,
         )
         if not block:
             result._inserted = inserted
@@ -575,3 +661,13 @@ class Table(DataFrame):
             if block
             else result
         )
+
+    def drop_table(self) -> None:
+        """Drops the table from the Snowflake database.
+
+        Note that subsequent operations such as :meth:`DataFrame.select`, :meth:`DataFrame.collect` on this ``Table`` instance and the derived DataFrame will raise errors because the underlying
+        table in the Snowflake database no longer exists.
+        """
+        self._session.sql(
+            f"drop table {self.table_name}"
+        )._internal_collect_with_tag_no_telemetry()
